<?php

namespace Drupal\field_highlightjs\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field_highlightjs\FieldHighlightHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Field highlightjs settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Drupal\field_highlightjs\FieldHighlightHelper service.
   *
   * @var \Drupal\field_highlightjs\FieldHighlightHelper
   */
  protected $helper;

  /**
   * Class constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory, FieldHighlightHelper $helper) {
    parent::__construct($config_factory);
    $this->helper = $helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('config.factory'),
      $container->get('field_highlight.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'field_highlightjs_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['field_highlightjs.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['#attached']['library'][] = 'field_highlightjs/init';
    $form['#attached']['library'][] = 'field_highlightjs/theme';
    $form['#attached']['library'][] = 'field_highlightjs/settings';

    $form['settings'] = [
      '#type' => 'container',
    ];
    $form['settings']['theme'] = [
      '#type' => 'select',
      '#title' => $this->t('Theme'),
      '#options' => $this->config('field_highlightjs.themes')->get('themes'),
      '#default_value' => $this->config('field_highlightjs.settings')->get('theme') ?? 'default',
    ];

    $versions = $this->helper->getVersions();
    $form['settings']['version'] = [
      '#type' => 'select',
      '#title' => $this->t('Version'),
      '#options' => $versions,
      '#default_value' => $this->config('field_highlightjs.settings')->get('version') ?? reset($versions),
    ];

    $form['settings']['cdn'] = [
      '#type' => 'select',
      '#title' => $this->t('CDN'),
      '#options' => [
        'cloudflare.com' => $this->t('cloudfare.com'),
        'jsdelivr.net' => $this->t('jsdeliver.net'),
      ],
      '#default_value' => $this->config('field_highlightjs.settings')->get('cdn') ?? reset($versions),
    ];

    $languages = $this->config('field_highlightjs.languages')->get('languages');
    $form['settings']['languages'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Languages'),
      '#description' => $this->t('Select the languages you wish to support. This selection will also alter your ckeditor5 settings for codeblocks!'),
      '#options' => $languages,
      '#default_value' => $this->config('field_highlightjs.settings')->get('languages') ?? $this->helper->getCommonLanguageKeys(),
    ];

    // Attach all languages.
    foreach (array_keys($languages) as $language) {
      $form['#attached']['library'][] = 'field_highlightjs/' . $language;
    }

    $form['preview'] = [
      '#type' => 'container',
    ];
    $form['preview']['preview_language'] = [
      '#type' => 'select',
      '#title' => $this->t('Preview language'),
      '#options' => $languages,
      '#default_value' => $form_state->getValue('preview_language') ?? 'php',
    ];
    $form['preview']['preview_markup'] = [
      '#type' => 'container',
      '#markup' => file_get_contents(__DIR__ . '/../../templates/preview.html'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('field_highlightjs.settings')
      ->set('languages', $form_state->getValue('languages'))
      ->set('theme', $form_state->getValue('theme'))
      ->set('version', $form_state->getValue('version'))
      ->set('cdn', $form_state->getValue('cdn'))
      ->save();

    // Maybe find a lighter way to refresh the libraries.
    drupal_flush_all_caches();

    parent::submitForm($form, $form_state);
  }

}
